@@ -9,10 +9,9 @@ use mro 'c3';
 
 use DBIx::Class::Carp;
 use Scalar::Util qw/refaddr weaken reftype blessed/;
-use List::Util qw/first/;
 use Context::Preserve 'preserve_context';
 use Try::Tiny;
-use SQL::Abstract qw(is_plain_value is_literal_value);
+use SQL::Abstract::Util qw(is_plain_value is_literal_value);
 use DBIx::Class::_Util qw(quote_sub perlstring serialize detected_reinvoked_destructor);
 use namespace::clean;
 
@@ -1061,6 +1060,12 @@ sub _populate_dbh {
 }
 
 sub _run_connection_actions {
+  # there are pathological cases in the CI where this can loop
+  # did not investigae in depth, but in either case this makes
+  # sense to guard like this
+  return if $_[0]->{_running_connections_actions};
+
+  local $_[0]->{_running_connections_actions} = 1;
 
   $_[0]->_do_connection_actions(connect_call_ => $_) for (
     ( $_[0]->on_connect_call || () ),
@@ -1114,6 +1119,9 @@ sub _server_info {
   unless ($info = $self->_dbh_details->{info}) {
 
     $info = {};
+    # this guarantees that problematic conninfo won't be hidden
+    # by the try{} below
+    $self->ensure_connected;
 
     my $server_version = try {
       $self->_get_server_version
@@ -1321,7 +1329,16 @@ sub _extract_driver_from_connect_info {
     # try to use dsn to not require being connected, the driver may still
     # force a connection later in _rebless to determine version
     # (dsn may not be supplied at all if all we do is make a mock-schema)
-    ($drv) = ($self->_dbi_connect_info->[0] || '') =~ /^dbi:([^:]+):/i;
+    #
+    # Use the same regex as the one used by DBI itself (even if the use of
+    # \w is odd given unicode):
+    # https://metacpan.org/source/TIMB/DBI-1.634/DBI.pm#L621
+    #
+    # DO NOT use https://metacpan.org/source/TIMB/DBI-1.634/DBI.pm#L559-566
+    # as there is a long-standing precedent of not loading DBI.pm until the
+    # very moment we are actually connecting
+    #
+    ($drv) = ($self->_dbi_connect_info->[0] || '') =~ /^dbi:(\w*)/i;
     $drv ||= $ENV{DBI_DRIVER};
   }
 
@@ -1371,24 +1388,40 @@ sub _warn_undetermined_driver {
 }
 
 sub _do_connection_actions {
-  my $self          = shift;
-  my $method_prefix = shift;
-  my $call          = shift;
+  my ($self, $method_prefix, $call, @args) = @_;
 
-  if (not ref($call)) {
-    my $method = $method_prefix . $call;
-    $self->$method(@_);
-  } elsif (ref($call) eq 'CODE') {
-    $self->$call(@_);
-  } elsif (ref($call) eq 'ARRAY') {
-    if (ref($call->[0]) ne 'ARRAY') {
-      $self->_do_connection_actions($method_prefix, $_) for @$call;
-    } else {
-      $self->_do_connection_actions($method_prefix, @$_) for @$call;
+  try {
+    if (not ref($call)) {
+      my $method = $method_prefix . $call;
+      $self->$method(@args);
+    }
+    elsif (ref($call) eq 'CODE') {
+      $self->$call(@args);
+    }
+    elsif (ref($call) eq 'ARRAY') {
+      if (ref($call->[0]) ne 'ARRAY') {
+        $self->_do_connection_actions($method_prefix, $_) for @$call;
+      }
+      else {
+        $self->_do_connection_actions($method_prefix, @$_) for @$call;
+      }
+    }
+    else {
+      $self->throw_exception (sprintf ("Don't know how to process conection actions of type '%s'", ref($call)) );
     }
-  } else {
-    $self->throw_exception (sprintf ("Don't know how to process conection actions of type '%s'", ref($call)) );
   }
+  catch {
+    if ( $method_prefix =~ /^connect/ ) {
+      # this is an on_connect cycle - we can't just throw while leaving
+      # a handle in an undefined state in our storage object
+      # kill it with fire and rethrow
+      $self->_dbh(undef);
+      $self->throw_exception( $_[0] );
+    }
+    else {
+      carp "Disconnect action failed: $_[0]";
+    }
+  };
 
   return $self;
 }
@@ -1445,6 +1478,92 @@ sub _do_query {
   return $self;
 }
 
+=head2 connect_call_rebase_sqlmaker
+
+This on-connect call takes as a single argument the name of a class to "rebase"
+the SQLMaker inheritance hierarchy upon. For this to work properly the target
+class B<MUST> inherit from L<DBIx::Class::SQLMaker::ClassicExtensions> and
+L<SQL::Abstract::Classic> as shown below.
+
+This infrastructure is provided to aid recent activity around experimental new
+aproaches to SQL generation within DBIx::Class. You can (and are encouraged to)
+mix and match old and new within the same codebase as follows:
+
+  package DBIx::Class::Awesomer::SQLMaker;
+  # you MUST inherit in this order to get the composition right
+  # you are free to override-without-next::method any part you need
+  use base qw(
+    DBIx::Class::SQLMaker::ClassicExtensions
+    << OPTIONAL::AWESOME::Class::Implementing::ExtraRainbowSauce >>
+    SQL::Abstract::Classic
+  );
+  << your new code goes here >>
+
+
+  ... and then ...
+
+
+  my $experimental_schema = $original_schema->connect(
+    sub {
+      $original_schema->storage->dbh
+    },
+    {
+      # the nested arrayref is important, as per
+      # https://metacpan.org/pod/DBIx::Class::Storage::DBI#on_connect_call
+      on_connect_call => [ [ rebase_sqlmaker => 'DBIx::Class::Awesomer::SQLMaker' ] ],
+    },
+  );
+
+=cut
+
+sub connect_call_rebase_sqlmaker {
+  my( $self, $requested_base_class ) = @_;
+
+  $self->throw_exception(
+    "The on_connect callee 'rebase_sqlmaker' expects a single plain string argument: the name of the target base class"
+  ) if (
+    @_ != 2
+      or
+    ! length( $requested_base_class )
+  );
+
+  my $old_class = ref( $self->sql_maker );
+
+  # nothing to do!
+  return if $old_class->isa( $requested_base_class );
+
+  my $synthetic_class = "${old_class}__REBASED_ON__${requested_base_class}";
+
+  {
+    no strict 'refs';
+
+    # skip if we already made that class
+    unless( @{"${synthetic_class}::ISA"} ) {
+
+      $self->ensure_class_loaded( $requested_base_class );
+
+      for my $base (qw(
+        DBIx::Class::SQLMaker::ClassicExtensions
+        SQL::Abstract::Classic
+      )) {
+
+        $self->throw_exception(
+          "The 'rebase_sqlmaker' target class '$requested_base_class' is not inheriting from '$base', this can not work"
+        ) unless $requested_base_class->isa( $base );
+      }
+
+      $self->inject_base( $synthetic_class, $old_class, $requested_base_class );
+
+      Class::C3->reinitialize
+        if DBIx::Class::_ENV_::OLD_MRO;
+    }
+  }
+
+  # force re-build on next access for this particular $storage instance
+  $self->sql_maker_class( $synthetic_class );
+  $self->_sql_maker( undef );
+}
+
 sub _connect {
   my $self = shift;
 
@@ -1659,7 +1778,7 @@ sub _gen_sql_bind {
       and
     $op eq 'select'
       and
-    first {
+    grep {
       length ref $_->[1]
         and
       blessed($_->[1])
@@ -1669,8 +1788,8 @@ sub _gen_sql_bind {
   ) {
     carp_unique 'DateTime objects passed to search() are not supported '
       . 'properly (InflateColumn::DateTime formats and settings are not '
-      . 'respected.) See "Formatting DateTime objects in queries" in '
-      . 'DBIx::Class::Manual::Cookbook. To disable this warning for good '
+      . 'respected.) See ".. format a DateTime object for searching?" in '
+      . 'DBIx::Class::Manual::FAQ. To disable this warning for good '
       . 'set $ENV{DBIC_DT_SEARCH_OK} to true'
   }
 
@@ -1700,7 +1819,6 @@ sub _resolve_bindattrs {
   };
 
   return [ map {
-    my $resolved =
       ( ref $_ ne 'ARRAY' or @$_ != 2 ) ? [ {}, $_ ]
     : ( ! defined $_->[0] )             ? [ {}, $_->[1] ]
     : (ref $_->[0] eq 'HASH')           ? [(
@@ -1717,31 +1835,6 @@ sub _resolve_bindattrs {
     :                                     [ $resolve_bindinfo->(
                                               { dbic_colname => $_->[0] }
                                             ), $_->[1] ]
-    ;
-
-    if (
-      ! exists $resolved->[0]{dbd_attrs}
-        and
-      ! $resolved->[0]{sqlt_datatype}
-        and
-      length ref $resolved->[1]
-        and
-      ! is_plain_value $resolved->[1]
-    ) {
-      require Data::Dumper;
-      local $Data::Dumper::Maxdepth = 1;
-      local $Data::Dumper::Terse = 1;
-      local $Data::Dumper::Useqq = 1;
-      local $Data::Dumper::Indent = 0;
-      local $Data::Dumper::Pad = ' ';
-      $self->throw_exception(
-        'You must supply a datatype/bindtype (see DBIx::Class::ResultSet/DBIC BIND VALUES) '
-      . 'for non-scalar value '. Data::Dumper::Dumper ($resolved->[1])
-      );
-    }
-
-    $resolved;
-
   } @$bind ];
 }
 
@@ -1975,12 +2068,30 @@ sub insert {
 
   my %returned_cols = %$to_insert;
   if (my $retlist = $sqla_opts->{returning}) {  # if IR is supported - we will get everything in one set
-    @ir_container = try {
-      local $SIG{__WARN__} = sub {};
-      my @r = $sth->fetchrow_array;
-      $sth->finish;
-      @r;
-    } unless @ir_container;
+
+    unless( @ir_container ) {
+      try {
+
+        # FIXME - need to investigate why Caelum silenced this in 4d4dc518
+        local $SIG{__WARN__} = sub {};
+
+        @ir_container = $sth->fetchrow_array;
+        $sth->finish;
+
+      } catch {
+        # Evict the $sth from the cache in case we got here, since the finish()
+        # is crucial, at least on older Firebirds, possibly on other engines too
+        #
+        # It would be too complex to make this a proper subclass override,
+        # and besides we already take the try{} penalty, adding a catch that
+        # triggers infrequently is a no-brainer
+        #
+        if( my $kids = $self->_dbh->{CachedKids} ) {
+          $kids->{$_} == $sth and delete $kids->{$_}
+            for keys %$kids
+        }
+      };
+    }
 
     @returned_cols{@$retlist} = @ir_container if @ir_container;
   }
@@ -2068,7 +2179,7 @@ sub _insert_bulk {
   # because a user-supplied literal/bind (or something else specific to a
   # resultsource and/or storage driver) can inject extra binds along the
   # way, so one can't rely on "shift positions" ordering at all. Also we
-  # can't just hand SQLA a set of some known "values" (e.g. hashrefs that
+  # can't just hand SQLMaker a set of some known "values" (e.g. hashrefs that
   # can be later matched up by address), because we want to supply a real
   # value on which perhaps e.g. datatype checks will be performed
   my ($proto_data, $serialized_bind_type_by_col_idx);
@@ -2416,7 +2527,7 @@ sub _select_args {
   # however currently we *may* pass the same $orig_attrs
   # with different ident/select/where
   # the whole interface needs to be rethought, since it
-  # was centered around the flawed SQLA API. We can do
+  # was centered around the flawed SQLMaker API. We can do
   # soooooo much better now. But that is also another
   # battle...
   #return (
